// ***************************************************************
//  madCodeHook.h             version:  1.3e    date: 2002-10-03
//  -------------------------------------------------------------
//  API hooking, code hooking
//  -------------------------------------------------------------
//  Copyright (C) 1999 - 2002 www.madshi.net, All Rights Reserved
// ***************************************************************

// 2002-10-03 1.3e (1) InjectLibraryW added
//                 (2) InjectLibrary2(W) added for use in CreateProcess(W) hooks
// 2002-09-22 1.3d CreateProcessExW added
// 2002-03-24 1.3c CollectHooks/FlushHooks speed up mixture initialization
// 2002-02-24 1.3b LPSTARTUPINFO -> LPSTARTUPINFOA
// 2002-01-21 1.3a ProcessHandleToID exported
// 2001-07-08 1.3  new functions (1) AllocMemEx & FreeMemEx
//                               (2) CopyFunction
//                               (3) CreateRemoteThread and
//                               (4) InjectLibrary added
// 2001-04-20 1.2a you can now force HookCode/API to use the mixture mode
// 2001-04-16 1.2  new function CreateProcessEx -> dll injecting

#ifdef __cplusplus

#define madCodeHookApi(rt) extern "C" __declspec(dllimport) rt WINAPI

// hook any code or a specific API
// under win9x you can hook code system wide, if it begins > $80000000
// for a system wide hook the callback function is automatically copied to shared memory
// use only kernel32.dll APIs in such a system wide hook callback function (!!)
madCodeHookApi(BOOL) HookCode(
  PVOID  pCode,
  PVOID  pCallbackFunc,
  PVOID  *pNextHook,
  PDWORD pdwErrorNo            = NULL,
  PSTR   pszErrorStr           = NULL,
  BOOL   bSystemWide           = FALSE,
  BOOL   bAcceptUnknownTargets = FALSE,
  BOOL   bForceMixtureMode     = FALSE
);
madCodeHookApi(BOOL) HookAPI(
  LPCSTR pszModule,
  LPCSTR pszFuncName,
  PVOID  pCallbackFunc,
  PVOID  *pNextHook,
  PDWORD pdwErrorNo            = NULL,
  PSTR   pszErrorStr           = NULL,
  BOOL   bSystemWide           = FALSE,
  BOOL   bAcceptUnknownTargets = FALSE,
  BOOL   bForceMixtureMode     = FALSE
);
madCodeHookApi(BOOL) UnhookCode(
  PVOID  *pNextHook
);

// memory allocation in the specified processes (shared memory in win9x)
// if the processHandle is 0, the memory is allocated or freed in the shared
// area (in win9x) or in our own process (in winNT)
madCodeHookApi(PVOID) AllocMemEx(
  DWORD  dwSize,
  HANDLE hProcess = NULL
);
madCodeHookApi(BOOL) FreeMemEx(
  PVOID  pMem,
  HANDLE hProcess = NULL
);

// copy (and relocate) any function to a new location in any process
// if the processHandle is 0, the function is copied to shared area (in win9x)
// or to another memory location in our own process (in winNT)
// don't forget to free the function with FreeMemEx, if you don't it anymore
madCodeHookApi(PVOID) CopyFunction(
  PVOID  pFunction,
  HANDLE hProcess              = 0,
  BOOL   bAcceptUnknownTargets = FALSE,
  PVOID  *pBuffer              = NULL
);

// this is how your remote function must look like
typedef DWORD (WINAPI *PREMOTE_EXECUTE_ROUTINE)( LPVOID pParams );

// executes the specified function in the context of another process
// this works only if the function follows some specific rules
// e.g. it must not use global variables, nor C++ private functions
// only win32 APIs are allowed
// if "dwSize" > 0, the "pParams" block will be copied to the other process
// after the remote function is finished, the "pParams" block is copied back
// so you can use the "pParams" block for both "in" and "out" parameters
// if "dwSize" = 0, the "pParams" value is just given into the remote function
madCodeHookApi(BOOL) RemoteExecute(
  HANDLE                  hProcess,
  PREMOTE_EXECUTE_ROUTINE pFunc,
  DWORD                   *dwFuncResult,
  PVOID			          pParams        = NULL,
  DWORD				      dwSize         = 0
);

#else

#define madCodeHookApi(rt) __declspec(dllimport) rt WINAPI

madCodeHookApi(BOOL) HookCode(
  PVOID  pCode,
  PVOID  pCallbackFunc,
  PVOID  *pNextHook,
  PDWORD pdwErrorNo,
  PSTR   pszErrorStr,
  BOOL   bSystemWide,
  BOOL   bAcceptUnknownTargets,
  BOOL   bForceMixtureMode
);
madCodeHookApi(BOOL) HookAPI(
  LPCSTR pszModule,
  LPCSTR pszFuncName,
  PVOID  pCallbackFunc,
  PVOID  *pNextHook,
  PDWORD pdwErrorNo,
  PSTR   pszErrorStr,
  BOOL   bSystemWide,
  BOOL   bAcceptUnknownTargets,
  BOOL   bForceMixtureMode
);
madCodeHookApi(BOOL) UnhookCode(
  PVOID  *pNextHook
);

madCodeHookApi(LPVOID) AllocMemEx(
  DWORD  dwSize,
  HANDLE hProcess
);
madCodeHookApi(BOOL) FreeMemEx(
  PVOID  pMem,
  HANDLE hProcess
);

madCodeHookApi(PVOID) CopyFunction(
  PVOID  pFunction,
  HANDLE hProcess,
  BOOL   bAcceptUnknownTargets,
  PVOID  *ppBuffer
);

typedef DWORD (WINAPI *PREMOTE_EXECUTE_ROUTINE)( LPVOID pParams );

madCodeHookApi(BOOL) RemoteExecute(
  HANDLE                  hProcess,
  PREMOTE_EXECUTE_ROUTINE pFunc,
  DWORD                   *dwFuncResult,
  PVOID			          pParams,
  DWORD				      dwSize
);

#endif

// putting all your "HookCode/API" calls into a "CollectHooks".."FlushHooks"
// frame can eventually speed up the installation of the hooks
madCodeHookApi(VOID) CollectHooks ();
madCodeHookApi(VOID)   FlushHooks ();

// same as CreateProcess
// additionally the dll "loadLibrary" is injected into the newly created process
// the dll is loaded right before the entry point of the exe module is called
madCodeHookApi(BOOL) CreateProcessExA(
  LPCSTR                lpApplicationName,
  LPSTR                 lpCommandLine,
  LPSECURITY_ATTRIBUTES lpProcessAttributes,
  LPSECURITY_ATTRIBUTES lpThreadAttributes,
  BOOL                  bInheritHandles,
  DWORD                 dwCreationFlags,
  LPVOID                lpEnvironment,
  LPCSTR                lpCurrentDirectory,
  LPSTARTUPINFOA        lpStartupInfo,
  LPPROCESS_INFORMATION lpProcessInformation,
  LPCSTR                lpLoadLibrary
);
madCodeHookApi(BOOL) CreateProcessExW(
  LPCWSTR               lpApplicationName,
  LPWSTR                lpCommandLine,
  LPSECURITY_ATTRIBUTES lpProcessAttributes,
  LPSECURITY_ATTRIBUTES lpThreadAttributes,
  BOOL                  bInheritHandles,
  DWORD                 dwCreationFlags,
  LPVOID                lpEnvironment,
  LPCWSTR               lpCurrentDirectory,
  LPSTARTUPINFOW        lpStartupInfo,
  LPPROCESS_INFORMATION lpProcessInformation,
  LPCWSTR               lpLoadLibrary
);

// like CreateRemoteThread, but 3 changes:
// (1) this one also works perfectly in win9x!!
// (2) this one also works on other sessions in winNt
// (3) the DACL of the current thread is copied in winNt (if threadAttr = nil)
madCodeHookApi(HANDLE) CreateRemoteThreadEx(
  HANDLE                 hProcess,
  LPSECURITY_ATTRIBUTES  lpThreadAttributes,
  DWORD                  dwStackSize,
  LPTHREAD_START_ROUTINE lpStartAddress,
  LPVOID                 lpParameter,
  DWORD                  dwCreationFlags,
  LPDWORD                lpThreadId
);

// (un)inject the specified dll into (from) all current and future processes
// can be used for InjectLibrary & UninjectLibrary
#define SYSTEM_WIDE  (HANDLE) 0xFFFFFFFD  // including system processes and other sessions
#define SESSION_WIDE (HANDLE) 0xFFFFFFFB  // only user application's of the current session

// same as LoadLibrary, but is able to load the library in any process
madCodeHookApi(BOOL) InjectLibraryA(
  HANDLE  hProcess,
  LPCSTR  pLibFileName
);
madCodeHookApi(BOOL) InjectLibraryW(
  HANDLE  hProcess,
  LPCWSTR pLibFileName
);

// stop the virus like injecting effect (can be used for UninjectLibrary only)
#define STOP_VIRUS (HANDLE) 0xFFFFFFFC

// same as FreeLibrary, but is able to free the library from any process
madCodeHookApi(BOOL) UninjectLibraryA(
  HANDLE  hProcess,
  LPCSTR  pLibFileName
);
madCodeHookApi(BOOL) UninjectLibraryW(
  HANDLE  hProcess,
  LPCWSTR pLibFileName
);

// which processID belongs to the specified process handle?
// undocumented function, works in all windows 32 bit systems
madCodeHookApi(DWORD) ProcessHandleToID(
  DWORD   dwProcessHandle
);

// is the current thread's desktop the input desktop?  (win9x -> always true)
// only in that case you should show messages boxes or other GUI stuff
madCodeHookApi(BOOL) AmUsingInputDesktop (VOID);

// is the specified process a service/system process?  (win9x -> always false)
madCodeHookApi(BOOL) IsSystemProcess (
  HANDLE  hProcess
);

#ifdef UNICODE

#define CreateProcessEx CreateProcessExW
#define   InjectLibrary   InjectLibraryW
#define UninjectLibrary UninjectLibraryW

#else

#define CreateProcessEx CreateProcessExA
#define   InjectLibrary   InjectLibraryA
#define UninjectLibrary UninjectLibraryA

#endif

