/***************************************************************************
    Shadow Door - A utility for NWN Extender to allow NWNScript to do AI
		in other services
    Copyright (C) 2003 Rob Zubek, Phillip Saltzman
    email: rob@cs.northwestern.edu, p-saltzman@northwestern.edu
		------------------------------------------------------------
				NWN Extender - Library for injection
    Copyright (C) 2003 Ingmar Stieger (Papillon)
    email: papillon@blackdagger.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <Winsock2.h>
#include <time.h>
#include "string.h"
#include "madCodeHookLib.h"
#include "ShadowDoor.h"
#include "msgparser.h"


FILE *logFile;
SOCKET externalSock;
bool connected;
bool verboseLog = false;
char logDir[8] = {0};
char toGameBuf[GAMEBUFSIZE] = {0};
HANDLE toGameMutex;
MsgParser parser(toGameBuf, toGameMutex);

void (*SetLocalStringNextHook)();

//printf to the logfile with a timestamp
void logPrintf(const char *fmt, ...) {
	va_list		ap;								// Pointer To List Of Arguments
	time_t t;
	tm* parsed_time;
	

	if (fmt == NULL)						// If There's No Text
		return;										
	time(&t);
	parsed_time = localtime(&t);
	fprintf(logFile, "***%d/%d/%d %.2d:%.2d:%.2d***\t", parsed_time->tm_mon + 1, parsed_time->tm_mday, parsed_time->tm_year + 1900,
																								parsed_time->tm_hour, parsed_time->tm_min, parsed_time->tm_sec);
	va_start(ap, fmt);						// Parses The String For Variables
  vfprintf(logFile, fmt, ap);		// And Converts Symbols To Actual Numbers
	va_end(ap);										// Results Are Stored In Text
}

void toggleVerboseLogging() {
	if (verboseLog)
		logPrintf("Stopped verbose logging\n");
	else
		logPrintf("Started verbose logging\n");
	verboseLog = !verboseLog;
}

void __declspec(naked) SetLocalStringHookProc()
{
	__asm {

		push ecx	  // save register contents
		push edx
		push ebx
		push esi
		push edi
		push ebp	  // prolog 1
        mov ebp, esp  // prolog 2

		mov eax, dword ptr ss:[esp+0x20]
		push eax
		mov eax, dword ptr ss:[esp+0x20]
		push eax
		call PayLoad
		add esp, 8

		pop ebp		// restore register contents
		pop edi		
		pop esi
		pop ebx
		pop edx
		pop ecx
		
		mov eax, dword ptr ss:[esp+0x8] // arg 2
		push eax
		mov eax, dword ptr ss:[esp+0x8] // arg 1
		push eax

		call SetLocalStringNextHook // call original function

		// cleanup stack
		pop eax
		add esp, 8
		push eax

        retn
	}
}

//Handle incomming requests from NWN
void PayLoad(char** str1, char** str2)
{
	if (!*str1 || !*str2)
		return;

	if (strncmp(*str1, "NWNX!SD!", 8) != 0) 	// not for us
		return;

	if (strncmp((*str1)+8, "SPACER", 6) == 0) 	// Ignore the buffer
		return;

	else if (connected) {
		if (strncmp((*str1) + 8, "SEND", 4) == 0) { //Message for the outside
			send(externalSock, *str2, strlen(*str2), 0);
			if (verboseLog) logPrintf("Sent to outside world: %s\n", *str2);
		}
		if (strncmp((*str1)+8, "READY?", 6) == 0) { //Request for a new incomming message
			parser.GetMessage(*str2);
			if (verboseLog) logPrintf("NWN requested message. Response: %s tokens\n", *str2);
		}
		else if (strncmp((*str1)+8, "GET-TOKEN", 8) == 0) { //Get the next token from the current message
			parser.NextToken(*str2);
			if (verboseLog) logPrintf("Sent NWN token: %s\n", *str2);
		}
		else if (strncmp((*str1)+8, "FINISH", 8) == 0) { //Release the message for NWN
			parser.Reset();
			if (verboseLog) logPrintf("NWN sent finish\n", *str2);
		}
		else
			*str2[0] = 0;
	}
	else
		*str2[0] = 0;
}

// returns log directory the server will use (logs.0, logs.1, ...)
char* GetLogDir()
{
	if (strlen(logDir) == 0)
	{
		int iDirectory = 0;
		char tmpNo[3] = {0};		
		char tmpFileName[26] = {0};
		HANDLE outFile;
		
		while (iDirectory < 100)
		{
			strcpy(tmpFileName, "logs.");
			itoa(iDirectory, tmpNo, 10);
			strcat(tmpFileName, tmpNo);
			strcat(tmpFileName, "\\nwserverlog1.txt");

			outFile = CreateFile(
			tmpFileName,						// pointer to name of the file
			GENERIC_READ | GENERIC_WRITE,       // access (read-write) mode
			0,									// exclusive mode
			NULL,                               // pointer to security attributes
			OPEN_ALWAYS,                        // how to create
			0,                                  // file attributes
			NULL);                              // handle to file with attributes to copy

			if (outFile != INVALID_HANDLE_VALUE)
			{
				// file not in use
				strcpy(logDir, "logs.");
				strcat(logDir, tmpNo);
				CloseHandle(outFile);
				break;
			}
			CloseHandle(outFile);
			iDirectory++;
		}
	}
	return logDir;
}

DWORD FindHook()
{
	char* ptr = (char*) 0x400000;
	while (ptr < (char*) 0x600000)
	{
		if ((ptr[0] == (char) 0x8b) &&
			(ptr[1] == (char) 0x44) &&
			(ptr[2] == (char) 0x24) &&
			(ptr[3] == (char) 0x04) &&
			(ptr[4] == (char) 0x6a) &&
			(ptr[5] == (char) 0x01) &&
			(ptr[6] == (char) 0x6a) &&
			(ptr[7] == (char) 0x03)
			)
			return (DWORD) ptr;
		else
			ptr++;

	}
	return NULL;
}

//Initialize the server socket for listening for connections
SOCKET initServerSock() {
	SOCKET servSock;
	sockaddr_in servAddr;
	int state = 0;

	//Listens for any incomming connection on port 1890
	servSock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	memset(&servAddr, 0, sizeof(servAddr));
	servAddr.sin_family=AF_INET;
	servAddr.sin_addr.S_un.S_addr = htonl(INADDR_ANY);
	servAddr.sin_port = htons(1890);
	state = bind(servSock, (sockaddr*)&servAddr, (sizeof(servAddr)));
	if (state < 0)
		logPrintf("bind returned error %d\n", WSAGetLastError());
	state = listen(servSock, 2);
	if (state < 0)
		logPrintf("listen returned error %d\n", WSAGetLastError());
	return servSock;
}

//Thread to deal with incomming connections and messages
DWORD WINAPI RunListener(LPVOID lpParam) 
{ 
	SOCKET servSock;
	char logFileName[18] = {0};
	int msgSize = 0;

	strcpy(logFileName, GetLogDir());
	strcat(logFileName, "\\nwnx.txt");

	logFile = fopen(logFileName, "wc");
	fprintf(logFile, "NWN Extender V.2.0.0.1\n");
	fprintf(logFile, "(c) 2003 by Ingmar Stieger (Papillon)\n");
	fprintf(logFile, "visit us at http://avlis.blackdagger.com\n\n");
	fprintf(logFile, "Shadow Door V. 0.9.0\n");
	fprintf(logFile, "by Phillip Saltzman and Robert Zubek\n\n");
	logPrintf("Shadow Door activated. Waiting for connections\n");

	servSock = initServerSock();
	connected = false;
	while(1) {
		externalSock = accept(servSock, NULL, NULL); //Wait for incomming connection
		connected = true;
		logPrintf("Shadow Door connected to outside world\n");
		do {
			recv(externalSock, NULL, 0, MSG_PEEK); //Wait for available data
			WaitForSingleObject(toGameMutex, INFINITE); //Set exclusive access to the buffer
			msgSize = recv(externalSock, toGameBuf, GAMEBUFSIZE, 0);
			if (msgSize >= 0 && msgSize < GAMEBUFSIZE) toGameBuf[msgSize] = 0;
			if (verboseLog) logPrintf("Recieved messgae from outside world: %s\n", toGameBuf);
			
			if (toGameBuf[0] == '@') {		//DLL command incomming
				if (toGameBuf[1] == 'v' || toGameBuf[1] == 'V')
					toggleVerboseLogging();
				toGameBuf[0] = 0; //Make sure NWN doesn't get the command meant for the DLL
			}
			ReleaseMutex(toGameMutex); //Release exclusive access;

		} while (msgSize > 0);
		logPrintf("External socket closed\n");
		connected = false;
		closesocket(externalSock);
   }

  return 0; 
} 

BOOL APIENTRY DllMain(HANDLE hModule, DWORD dwReason, LPVOID lpReserved)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		DWORD hookAt = FindHook();
		HookCode((PVOID) hookAt, SetLocalStringHookProc, (PVOID*) &SetLocalStringNextHook);

		//Create mutex for the toGameBuffer
		toGameMutex = CreateMutex(NULL, FALSE, NULL);
		parser.setMutex(toGameMutex);

		//Start Winsock
		WSADATA wsData;
		WORD    wVer = MAKEWORD(2,2);
		int status = WSAStartup(wVer,&wsData);
		if (status < 0) logPrintf("Error initializing winsock\n");

		//Run the lister thread
		DWORD dwThreadId;
		HANDLE hThread; 
		hThread = CreateThread(NULL, 0, RunListener, NULL, 0, &dwThreadId);                
		CloseHandle( hThread );
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		logPrintf("NWNX2 shutting down.\n");
		fclose(logFile);
		WSACleanup();
	}

	return TRUE;
}