;;
;;
;; shadow door action interface
;;
;; some simple functions that handle the different types of actions
;; supported by the shadow door interface

(in-package :common-lisp-user)



;;
;;
;; recognizing received commands

;; attack helpers

(defun sd-attack? (sexp)
  (equal (first sexp) "attacked-by"))

(defun sd-attacker-name (sexp)
  (second sexp))

;; speech helpers

(defun sd-speech? (sexp)
  (equal (first sexp) "speech"))

(defun sd-speech-speaker (sexp)
  (second sexp))

(defun sd-speech-emote (sexp)
  (third sexp))

(defun sd-speech-text (sexp)
  (fourth sexp))



;;
;;
;; sending commands


;; sends a speak! command, to output a given text string.
;; emote is a string such as "*shouts*", can be used for emotive
;; expressions, onomatopoeia, etc.

(defun sd-send-speak (text &optional (emote ""))
  (sd-write-sexp `(speak! ,emote ,text)))


;; sends an attack! command, to attack the named player character.
;; passive? flag determines whether to abandon the attack if the 
;; target is not immediately at hand.

(defun sd-send-attack (target-tag &optional (passive? nil))
  (sd-write-sexp `(attack! ,target-tag ,passive?)))


;; sends a moveto! command, to approach the named object.
;; run? is a self-explanatory flag; range specifies how close 
;; to approach the object

(defun sd-send-moveto (target-tag &optional (run? nil) (range 1.0))
  (sd-write-sexp `(moveto! ,target-tag ,run? ,(format nil "~3,1F" range))))


;; sends an animate! command, to perform a stored animation
;; for animation-constant, see below. speed is the playback speed
;; (where 1.0 is normal), and duration-secs is the duration of 
;; playback for looping animation types

(defun sd-send-animate (animation-constant &optional (speed 1.0) (duration-secs 0.0))
  (sd-write-sexp `(animate! ,(format nil "~D" animation-constant)
                            ,(format nil "~3,1F" speed)
                            ,(format nil "~3,1F" duration-secs))))

;;
;;
;; animation constants for the animate! command
;;
;; nota bene: not all characters support all animation types!

;; looping animations

(defconstant animation-looping-pause 0)
(defconstant animation-looping-pause2 1)
(defconstant animation-looping-listen 2)
(defconstant animation-looping-mediate 3)
(defconstant animation-looping-worship 4)
(defconstant animation-looping-look-far 5)
(defconstant animation-looping-sit-chair 6)
(defconstant animation-looping-sit-floor 7)
(defconstant animation-looping-talk-normal 8)
(defconstant animation-looping-talk-pleading 9)
(defconstant animation-looping-talk-forceful 10)
(defconstant animation-looping-talk-laughing 11)
(defconstant animation-looping-get-low 12)
(defconstant animation-looping-get-mid 13)
(defconstant animation-looping-pause-tired 14)
(defconstant animation-looping-pause-drunk 15)

;; fire and forget animations

(defconstant animation-oneshot-head-turn-left 100)
(defconstant animation-oneshot-head-turn-right 101)
(defconstant animation-oneshot-pause-scratch-head 102)
(defconstant animation-oneshot-pause-bored 103)
(defconstant animation-oneshot-salute 104)
(defconstant animation-oneshot-bow 105)
(defconstant animation-oneshot-steal 106)
(defconstant animation-oneshot-greeting 107)
(defconstant animation-oneshot-taunt 108)
(defconstant animation-oneshot-victory1 109)
(defconstant animation-oneshot-victory2 110)
(defconstant animation-oneshot-victory3 111)
(defconstant animation-oneshot-read 112)
(defconstant animation-oneshot-drink 113)



